@@ -3,20 +3,37 @@ Support for rendering in a Jupyter widget. Provides a widget subclass that
 can be used as cell output, or embedded in a ipywidgets gui.
 """
 
+import weakref
+import asyncio
+
 from ._offscreen import WgpuOffscreenCanvas
 
 import numpy as np
 from jupyter_rfb import RemoteFrameBuffer
+from IPython.display import display
+
+
+pending_jupyter_canvases = []
 
 
 class JupyterWgpuCanvas(WgpuOffscreenCanvas, RemoteFrameBuffer):
     """An ipywidgets widget providing a wgpu canvas. Needs the jupyter_rfb library."""
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, *, size=None, title=None, **kwargs):
+        super().__init__(**kwargs)
+
+        # Internal variables
         self._pixel_ratio = 1
         self._logical_size = 0, 0
         self._is_closed = False
+        self._request_draw_timer_running = False
+
+        # Register so this can be display'ed when run() is called
+        pending_jupyter_canvases.append(weakref.ref(self))
+
+        # Initialize size
+        if size is not None:
+            self.set_logical_size(*size)
 
     # Implementation needed for RemoteFrameBuffer
 
@@ -29,6 +46,7 @@ class JupyterWgpuCanvas(WgpuOffscreenCanvas, RemoteFrameBuffer):
             self._logical_size = event["width"], event["height"]
 
     def get_frame(self):
+        self._request_draw_timer_running = False
         # The _draw_frame_and_present() does the drawing and then calls
         # present_context.present(), which calls our present() method.
         # The resuls is either a numpy array or None, and this matches
@@ -59,7 +77,9 @@ class JupyterWgpuCanvas(WgpuOffscreenCanvas, RemoteFrameBuffer):
         return self._is_closed
 
     def _request_draw(self):
-        RemoteFrameBuffer.request_draw(self)
+        if not self._request_draw_timer_running:
+            self._request_draw_timer_running = True
+            call_later(self._get_draw_wait_time(), RemoteFrameBuffer.request_draw, self)
 
     # Implementation needed for WgpuOffscreenCanvas
 
@@ -94,3 +114,18 @@ class JupyterWgpuCanvas(WgpuOffscreenCanvas, RemoteFrameBuffer):
 
 # Make available under a name that is the same for all gui backends
 WgpuCanvas = JupyterWgpuCanvas
+
+
+def call_later(delay, callback, *args):
+    loop = asyncio.get_event_loop()
+    loop.call_later(delay, callback, *args)
+
+
+def run():
+    # Show all widgets that have been created so far.
+    # No need to actually start an event loop, since Jupyter already runs it.
+    canvases = [r() for r in pending_jupyter_canvases]
+    pending_jupyter_canvases.clear()
+    for w in canvases:
+        if w and not w.is_closed():
+            display(w)
